% PROGRAM     :  SHAPE [Seismic HAzard Parameters Evaluation]  
% VERSION     :  V_2b.1 [Wrapper (fast) Standalone Version] 
% LAST UPDATED:  June 2020
% COMPATIBLE  :  Matlab version 2017b or later
% REQUIREMENTS:  "Statistics and Machine Learning" Matlab Toolbox installed
% TOOLBOX     :  "Hazard Analysis Toolbox" developed within SERA Project
% USER GUIDE  :  "READ_ME_SHAPE_ver2b.pdf"
% CITE AS     :  Leptokaropoulos K. and S. Lasocki (2020), Seismol. Res. Lett., doi: 10.1785/0220190319 
% CONTACT     :  kleptoka@igf.edu.pl (Dr. Kostas Leptokaropoulos)
% --------------------------------------------------------------------------------------------------------------------
% Time-and-Technology Dependent Seismic Hazard Assessment (SHA)  
% --------------------------------------------------------------------------------------------------------------------
% INPUT:
% !!! ----------------------------  INPUT DATA REQUIREMENTS ----------------------------- !!!
% the program works with ASCII input data files (e.g. *.txt). The files needed are:
% >   File with the parameters of seismic data [mandatory]
% >   File with the parameters of production data [optional]
% >   File specifying time windows for SHA analysis [optional]
% >   File with the fields description of the corresponding parameters in the seismic data file
% >   File with the fields description of the corresponding parameters in the production data files
% FOR DETAILS on data requirements please refer to the document:
%                            "READ_ME_SHAPE_ver2b.pdf"
% --------------------------------------------------------------------------------------------------------------------
% OVERVIEW: THE PROGRAM takes as input a Seismic and optionally, a Production data parameter file to provide Seismic 
% Hazard Assessment for specified time-windows, following User's specifications. 
% --------------------------------------------------------------------------------------------------------------------
% AUTHORS: K. Leptokaropoulos and S. Lasocki
% Based on magnitude distribution/stationary hazard parameters estimation functions originally developed by S. Lasocki
% Last Updated: 06/2020, within SERA PROJECT, EU Horizon 2020 R&I programme under grant agreement No.730900
% CURRENT VERSION: v2b.1        ****    [WRAPPER (fast) STANDALONE VERSION!!]
%% - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
% PLEASE refer to the accompanying document:
%                                           "READ_ME_SHAPE_ver2b.pdf" 
% for description of the Application and its requirements.
%% -----------------------------------------------------------------------------------------------------------------------
% DESCRIPTION: The Application performs time-dependent Seismic Hazard Analysis (SHA), 
% taking into account the activity rate and the magnitude distribution of seismicity 
% for selected time windows. The hazard parameters estimated are: 
%    1) The Mean Return Period (MRP) of a given magnitude, M, which is defined as the 
%       average elapsed time between the occurrence of consecutive events of M and 
%    2) The Exceedance Probability (EPR) of a given magnitude, M, within a given time 
%       period of length, T, which is defined as the probability of an earthquake of 
%      M to occur during T.  
% These hazard parameters together with the 95% confidence interval are estimated for 
% different time windows which are constructe upon Users particular specifications. 
% 4 different magnitude distribution models can be chosen. The input files must be in 
% ASCII format (e.g. *.txt). A brief description of the preparation process is given here:
% !!! THE USER SETS THE PARAMETERS IN LINES 125-143 !!! then the following steps are executed
% (please see in the script for more comments and details for each STEP)
% STEP_1. Upload Data
% STEP_2. Select Magnidue Scale
% STEP_3. Filter Data for Mc 
% STEP_4. TIME WINDOWS GENERATION
% STEP_5. SHA PARAMETERS ESTIMATION
% STEP_6. Plotting (optional)
% STEP_7. save OUTPUTS
% ---------------------------------------------------------------------------------------------------------------------
%% INPUT:    All input data are sufficiently explained in the script as well as while running the code 
%               (interaction with the user). NOTE that all input files (seismic catalog, production data, 
%               time windows) must be in ASCII format (i.e. *.txt). Please refer to the APPLICATION DOCUMENTATION 
%               for further instructions and input data requirement specifications:  "READ_ME_SHAPE_ver2b.pdf"
% ----------------------------------------------------------------------------------------------------------------------
%% OUTPUT:
%                 <> Output Report with summary of the Results as well as data and parameters used 
%                 <> Output Figure with the results in *.mat and *.jpeg formats (optional)
%                 <> Output Matlab Structure with input parameter values and output results, having
%                    as many cells as the number of time windows generated.
%                                 Structure fields are:
%                    - Time      : vector with origin times of the events included in each time window
%                    - M         : vector with events magnitudes
%                    - Mmin      : Completeness magnitude
%                    - eps       : Magnitude round-off interval
%                    - lambd     : mean activity rate
%                    - lambd_err : events number sufficiency (0-all parametes estiamated, 1-all parameters set as NaNs)
%                    - unit      : Time Unit
%                    - method    : Magnitude Distribution Model
%                    - b         : b-value of GR law    
%                                  [applies only when "method" is set to 'GRU' or 'GRT']
%                    - b_0025    : these two parameters define the b-value 95% bootstrap Confidence interval
%                    - b_0075    :                           -//-
%                    - h         : Kernel smoothing factor 
%                                  [applies only when "method" is set to 'NPU' or 'NPT']
%                    - xx        : Background sample for kernel magnitude estimate 
%                                  [applies only when "method" is set to 'NPU' or 'NPT']
%                    - ambd      : weigthing factors for the adaptive kernel 
%                                  [applies only when "method" is set to 'NPU' or 'NPT']
%                    - ierr      : h convergence indicator (0-converges,1-multiple zeros, 2-no zeros) 
%                                  [applies only when "method" is set to 'NPU' or 'NPT']
%                    - Mmax      : Upper limit of magnitude distribution (truncated) 
%                                  [applies only when "method" is set to 'GRT' or 'NPT']
%                    - err       : Mmax convergence indicator (0-converge, 1-no converge) 
%                                  [applies only when "method" is set to 'GRT' or 'NPT']
%                    - PDF       : 2 columns, first representing magnitudes and second the Probability Density Function of those magnitudes
%                    - CDF       : 2 columns, first representing magnitudes and second the Cumulative Distribution Function of those magnitudes
%                    - MRP       : Mean Return Period 
%                    - MRP_0.025 : these two parameters define the MRP 95% bootstrap Confidence interval
%                    - MRP_0.975 :                           -//-
%                    - EP        : Exceedance Probability                         
%                    - EP_0.025  : these two parameters define the EP 95% bootstrap Confidence interval
%                    - EP_0.975  :                           -//-
% -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- 
% REFERENCES:
% Kijko A, Lasocki S, Graham G (2001), Pure Appl. Geophys. 158:16551676
% Kijko A, Sellevoll MA (1989), Bull Seismol. Soc. Am. 79:645654
% Lasocki S (2017), Chapter 11.3 in Rockburst Mechanisms, Oxford, pp 366-380
% Lasocki S, Urban P (2011), Acta Geophys. 59:659673
% Lasocki S, Orlecka-Sikora B (2008), Tectonophysics 456:2837
% Leptokaropoulos K, Staszek M, Cielesta S, Urban P, Olszewska D, Lizurek G (2017), Acta Geophys. 65:493-505 
% Leptokaropoulos K, Lasocki S (2020), Seismol. Res. Lett., doi: 10.1785/0220190319.
% ---------------------------------------------------------------------------------------------------------------------
% LICENSE 
%     This is free software: you can redistribute it and/or modify it under the terms of the 
%     GNU General Public License as published by the Free Software Foundation, either version 
%     3 of the License, or (at your option) any later version.
%     This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
%     without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
%     See the GNU General Public License for more details.
% -------------------------------------------------------------------------------------------------------------------

clc;clear; 
close all;mkdir Outputs_SHA

% PLEASE SET INPUT ARGUMENTS [LINES 125-143]
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SEIS_DATA='ST2_SEIS_Data.txt';               % Seismic Data File - NOTE: SEIS_DATA=[] is valid as well
SEIS_FIELDS='ST2_SEIS_Fields.txt';           % Seismic Data Fields File
PROD_DATA='ST2_PROD_Data.txt';               % Production Data (non-seismic) File - NOTE: PROD_DATA=[] is valid as well
PROD_FIELDS='ST2_PROD_Fields.txt';           % Production Data (non-seismic) Fields File
PROD_FIELD=2;                                % field (column) corresponding to a selected Production Parameter  
MScale='ML';                                 % Magnitude Scale (e.g. 'ML', 'Mw' etc). NOTE: MScale=[] is also valid
Mc=1.0;                                      % Select Mc
Mmax=[];                                     % Valid for GRT and NPT/ if Mmax=[], it is calculated internally
Nsynth=1000;                                 % number of trials to estimated Mmax bias/ if Nsynth=[], no bias is estimated
winmode='Time';                              % Select MODE for windows creation: 'Time' or 'Events' or 'File'
file_n='ST2_test_timewindows.txt';           % Select file name with starting and ending time of time windows, applicable only for winmode='File'
window_size=50;                              % time window span (days or events, depending on "winmode")
dt=50;                                       % time step (days)
method='GRT';                                % Select M distribution model among 'GRU','GRT','NPU','NPT'
Tunit='day';                                 % Select time unit among 'day', 'month', 'year'
MaG=3.0;                                     % set target Magnitude for EPR and MRP calculation
Plength=1;                                   % set target time Period (days) for EPR calculation
Nbst=100;                                     % number of bootstrap iterations to determine Hazard Parameters confidence interval
Plotopt='ON';                               % To enable ('ON') or disable ('OFF') plotting
%% -------------------------------------------------------------------------------------------------------------------------

if method=='GRT' & isempty(Mmax)==0 & MaG>Mmax | method=='NPT' & isempty(Mmax)==0 & MaG>Mmax 
error('Mmax is smaller than the target magnitude (MaG), please change input')
end

%% STEP 1:  Load data
[Catalog,PROD_Data,s1]=Data_Hand_A2M_ver2...
    (SEIS_DATA,SEIS_FIELDS,PROD_DATA,PROD_FIELDS,PROD_FIELD);

%% STEP 2: Select Magnitude Scale
[Ctime,Cmag]=Select_Magnitude_Scale_ver2(Catalog,MScale);

%% STEP 3: Filter data for Mc
[Ctime,Cmag,Catalog]=FiltMc_ver2(Ctime,Cmag,Catalog,s1,Mc);

%% STEP 3b: Estimate Mmax for Truncated Distributions (GRT and NPT)
cd SSH/
if strcmp(method,'GRT')==1 & isempty(Mmax)==1;
[p1,p2,p3,p4,p5,pb,Mmax,err,BIAS,SD]=TruncGR_O(Ctime,Cmag,0,Mc,Mmax,Nsynth);    
Mmax=Mmax+BIAS;
elseif strcmp(method,'NPT')==1 & isempty(Mmax)==1;
[p1,p2,p3,p4,p5,p6,p7,p8,p9,Mmax,err,BIAS,SD]=Nonpar_tr_O(Ctime,Cmag,0,Mc,Mmax,Nsynth);
Mmax=Mmax+BIAS;
end

cd ../

%% STEP 4: Create Time Windows
time_windows=struct;time_windows.Time=[];
 to=Ctime-Ctime(1);tmin=min(to);tmax=max(to);

switch winmode
%%    TIME 
    case 'Time'
        if window_size>tmax;n=1;warning('time window is set larger than data time span');end
        n=ceil((tmax-window_size)/dt);
        for i=1:n
        time_windows(i).Time=Ctime(to>=(i-1)*dt & to<(i-1)*dt+window_size);
        time_windows(i).M=Cmag(to>=(i-1)*dt & to<(i-1)*dt+window_size);
        time_windows(i).Tstart=Ctime(1)+(i-1)*dt;
        time_windows(i).Tend=Ctime(1)+(i-1)*dt+window_size;
        end  
%%    EVENTS
    case 'Events'
 if window_size>numel(to);window_size=numel(to);warning('events window is set larger than given data');end    
   n=ceil(tmax/dt);
        for i=1:n
        To=find(to>=(i-1)*dt);To=To(1);
        if To<=length(Ctime)-window_size+1;
        time_windows(i).Time=Ctime(To:To+window_size-1);
        time_windows(i).M=Cmag(To:To+window_size-1);
        time_windows(i).Tstart=Ctime(1)+(i-1)*dt;
        time_windows(i).Tend=max(time_windows(i).Time);
        end     
        end
 %%    FILE
    case 'File'
    cd TIME_WINDOWS
    Twindows=dlmread(file_n);
    T1=Twindows(:,1);T2=Twindows(:,2);n=numel(T1);
    for i=1:n
       time_windows(i).Time=Ctime(Ctime>=T1(i) & Ctime<T2(i));
       time_windows(i).M=Cmag(Ctime>=T1(i) & Ctime<T2(i));
       time_windows(i).Tstart=T1(i);
       time_windows(i).Tend=T2(i);
    end
    cd ../
end
 

%% STEP 5: ESTIMATE HAZARD PARAMETERS
if strcmp(Tunit,'day');iop=0;elseif strcmp(Tunit,'month');iop=1;elseif strcmp(Tunit,'year');iop=2;
else; error('Please set "Tunit" Parameter as "day", "month" or "year"');end

%% RUN MAGDIST
[HP] = TDHMagDistWrapper(method, time_windows, Mc, iop,Mmax,Nbst);

%% Harzard Parameters Estimate
     %[MRPer,ExPr,MRPer_low,MRPer_high,ExPr_low,ExPr_high]=TDHRetPeriodExcProbWrapper(method,MaG,Plength,Mc,HP,Plength)
 for i=1:length(HP)
     if isnan(HP(i).CDF)
      lambda(i)=HP(i).lamb;MRPer(i)=NaN;ExPr(i)=NaN;MRPer_low(i)=NaN;MRPer_high(i)=NaN;ExPr_low(i)=NaN;ExPr_high(i)=NaN;
     else
 [MRPer1,ExPr1,MRPer_low1,MRPer_high1,ExPr_low1,ExPr_high1]=RetPeriodExcProbWrapper(HP(i).lamb,HP(i).CDF,Plength,MaG);
 lambda(i)=HP(i).lamb;
 MRPer(i)=MRPer1;ExPr(i)=ExPr1;
 MRPer_low(i)=MRPer_low1;MRPer_high(i)=MRPer_high1;
 ExPr_low(i)=ExPr_low1;ExPr_high(i)=ExPr_high1;
     end
 end
%% 
%% STEP 7: Ploting (optional)
Zplo_ver2b_1

%% STEP 6: Save outputs
Zsave_output_ver2b_1


%% -*-*-*-*-*-*-*-*-*-*-*-*-*- F U N C T I O N S -*-*-*-*-*-*-*-*-*-*-*-*-*-
   
%% ------------------------- DATA HANDLING FUNCTION   --------------------------

function [Catalog,PROD_data,ss1]=Data_Hand_A2M_ver2...
    (SEIS_DATA,SEIS_FIELDS,PROD_DATA,PROD_FIELDS,PROD_FIELD)

if isempty(PROD_DATA)
%% SEISMIC DATA
cd CATALOGS
 SData=load(SEIS_DATA);
 SFields=fileread(SEIS_FIELDS);
cd ../

Na=length(SFields);if SFields(Na)~=' ';SFields(Na+1)=' ';end
[cou,c,Datime,Catalog]=Fields_dat(SData,SFields,1);
ss1=1:length(Catalog);% ss1=SetParams(Catalog);%Catalog=Catalog(ss1);
PROD_data=[];ss=[];s2=[];ss2=[];dstr2=[];

else
%% ------------------------------------------------------------------------------------------------------------------------------------
% BOTH -SEISMIC and PRODUCTION DATA

cd CATALOGS                                             %Seismic Data
 SData=load(SEIS_DATA);
 SFields=fileread(SEIS_FIELDS);
cd ../

Na=length(SFields);if SFields(Na)~=' ';SFields(Na+1)=' ';end
[cou,c,Datime,Catalog]=Fields_dat(SData,SFields,1);
ss1=1:length(Catalog);
% ss1=SetParams(Catalog);%Catalog=Catalog(ss1);
 
cd PRODUCTION_DATA                         % Production (non-Seismic) Data
OData=load(PROD_DATA);
OFields=fileread(PROD_FIELDS);
cd ../

Na1=length(OFields);if OFields(Na1)~=' ';OFields(Na1+1)=' ';end
[cou1,c1,Datime1,PROD_data]=Fields_dat(OData,OFields,2);

end
 % save('Catalog_ST2_Test','Catalog')
 % save('Catalog_ST2_Test','Catalog')


%% -----------------------------------------    F U N C T I O N S    -----------------------------------------
function [cou,c,Datime,OUT]=Fields_dat(indata,infields,iop)

Datime=datenum(indata(:,1),indata(:,2),indata(:,3),indata(:,4),indata(:,5),indata(:,6));       % Convert time to matlab format

% Define Fields
c=1;
for i=1:length(infields)-1
if strcmp(infields(i),' ')==1;cou(i)=0;
else cou(i)=c;end
if strcmp(infields(i),' ')==0 & strcmp(infields(i+1),' ')==1;c=c+1;end
end

if iop==1
OUT(1).field='Occurrence_Time';OUT(1).val=Datime;
elseif iop==2
OUT(1).field='Production_Time';OUT(1).val=Datime;
end

for i=2:c-1
OUT(i).field=infields(cou==i);    
OUT(i).val=indata(:,i+5);
end

%Set Field Type for Magnitude Recognition
for i=1:size(OUT,2)
    if strcmp(OUT(i).field,'ML') || strcmp(OUT(i).field,'Mw') || strcmp(OUT(i).field,'M') ...
   || strcmp(OUT(i).field,'Ms') || strcmp(OUT(i).field,'mb') || strcmp(OUT(i).field,'Md')
         OUT(i).fieldType='Magnitude';
    else
    OUT(i).fieldType=[];
    end
end

end

%% -----------------------------------------------------------------------------------------------------------


% ----------------------------------------------------------------------------------------------------------------


end

%%
function [Ctime,Cmag,Data]=FiltMc_ver2(Ctime,Cmag,Catalog,s1,Mc)
clc
id_time=findfield(Catalog,'Occurrence_Time');
                                                                            %opts.Interpreter='tex';opts.Default='Yes';
                                                                            %quest='Do you wish to filter Data for Mc?';
                                                                            %answer=questdlg(quest,'Data Completenes','Yes','No',opts);
                                                                            %if strcmp(answer,'Yes')

%% THIS HAS MOVED TO A SEPARATE FUNCTION IN THE BEGINNING OF THE APPLICATION
% cou=1;
% for i=1:length(Catalog)
%     if strcmp(Catalog(i).fieldType,'Magnitude')==1
%         C(cou).field=Catalog(i).field;cou=cou+1;
%     end
% end
% 
% % Check for no magnitude
% if cou==1;error('MyComponent:incorrectType',...
% 'No magnitude column detected!! Please check: Magnitude fields must be noted as one of the following:\nM  ,  Mw  ,  ML  ,  Ms  ,  mb  ,  Md\n or select the entire sample for analysis');end
% 
% %Select Parameters from Seismic Catalog -
% [ss1,ok]=listdlg('PromptString','Please Select M scale:',...
%     'ListString',{C.field}, 'SelectionMode','single');
% 
%  id=findfield(Catalog,C(ss1).field);
%   Mtype=Catalog(id).field;
% Ctime=Catalog(id_time).val;
% id_M=findfield(Catalog,Mtype);
% Cmag=Catalog(id_M).val;

%% 

cou=1;


for i=1:length(s1)
    x=Catalog(s1(i)).val;x=x(Cmag>=Mc);
    index=isnan(x)==0;
    x=x(isnan(x)==0);
    Data(cou).field=Catalog(s1(i)).field;
    Data(cou).fieldType=Catalog(s1(i)).fieldType;
    Data(cou).val=nan(size(index)); Data(cou).val(index)=x;
    cou=cou+1;
end

Ctime=Ctime(Cmag>=Mc);Cmag=Cmag(Cmag>=Mc);

n=length(Ctime);
disp(['number of events: ',num2str(n)]) 

end

%% --------------------------------------------------------------------------------------
% finds a field defined by a certain (string) name
function [id] = findfield( catalog,field )
id=0;
j=1;
while j <= size(catalog,2) && id==0
        if (strcmp(catalog(j).field,field)==1)
        id=j;
        end
        j=j+1;
end
end

%% 
function [Ctime,Cmag]=Select_Magnitude_Scale_ver2(Catalog,MType)

cou=1;
for i=1:length(Catalog)
    if strcmp(Catalog(i).fieldType,'Magnitude')==1
        C(cou).field=Catalog(i).field;cou=cou+1;
    end
end

% Check for no magnitude
if cou==1;error('MyComponent:incorrectType',...
'No magnitude column detected!! Please check: Magnitude fields must be noted as one of the following:\nM  ,  Mw  ,  ML  ,  Ms  ,  mb  ,  Md\n or select the entire sample for analysis');end


 id=findfield(Catalog,MType);
 id_time=findfield(Catalog,'Occurrence_Time');
Cmag=Catalog(id).val;Ctime=Catalog(id_time).val;

end

%% -----------!!!!!!!!!!! HAZARD PARAMETERS ESTIMATE FUNCTIONS !!!!!!!!!!!-----------

function [HP] = TDHMagDistWrapper(method, time_win_data, mmin, iop,Mmax,Nbst)
cd SSH
for j=1:size(time_win_data,2);if isempty(time_win_data(j).Time);M1(j)=NaN;else;M1(j)=max(time_win_data(j).M);end;end
Mmaxcat=max(M1);Mmax=max([Mmaxcat Mmax])

	for i=1:size(time_win_data,2)
		mags_vec = time_win_data(i).M;
		time_vec = time_win_data(i).Time;
		HP(i).mmin = mmin;
		HP(i).iop = iop;
		HP(i).method = method;
		switch method
			case 'GRU'
				try
					[HP(i).lamb_all, HP(i).lamb, HP(i).lamb_err, HP(i).unit, HP(i).eps, HP(i).b, HP(i).bCI]=UnlimitGR(time_vec, mags_vec, iop, mmin,Nbst);
                    HP(i).Mmax=NaN;eps=HP(i).eps;
                    [m, PDF_GRU, CDF_GRU]=dist_GRU_CI(mmin,Mmax+eps,eps,mmin,eps,HP(i).b,HP(i).bCI);                        %K29JAN2020 - magnitude PDF/CDF 
                    HP(i).PDF=[m PDF_GRU];HP(i).CDF=[m CDF_GRU];                                                         %K29JAN2020 - magnitude PDF/CDF 
				catch err
	                HP(i).lamb_all=NaN; HP(i).lamb=NaN; HP(i).lamb_err=2; HP(i).unit=''; HP(i).eps=NaN; HP(i).b=NaN;HP(i).Mmax=NaN;HP(i).PDF=NaN;HP(i).CDF=NaN;
	                HP(i).bCI(1)=NaN;HP(i).bCI(2)=NaN;warning('%s: %s', err.identifier, err.message);
				end
			case 'GRT'
				try
					[HP(i).lamb_all, HP(i).lamb, HP(i).lamb_err, HP(i).unit, HP(i).eps, HP(i).b, HP(i).Mmax, HP(i).err]=TruncGR_O(time_vec, mags_vec, iop, mmin,Mmax);
                    eps=HP(i).eps;[m, PDF_GRT, CDF_GRT]=dist_GRT(mmin,Mmax+eps,eps,mmin,eps,HP(i).b,Mmax);                    %K29JAN2020 - magnitude PDF/CDF 
                    funct=@(mags_vec)TruncGR_O_CI(time_vec,mags_vec,iop,mmin,Mmax,[]);                      % K04JUN2020 GRT
                    HP(i).bCI=bootci(Nbst,{funct,mags_vec},'alpha',0.05);                                   % K04JUN2020 GRT
                    [m, PDF_GRT, CDF_GRT]=dist_GRT_CI(mmin,Mmax+eps,eps,mmin,eps,HP(i).b,Mmax,HP(i).bCI);   % K04JUN2020 GRT
                    HP(i).PDF=[m PDF_GRT];HP(i).CDF=[m CDF_GRT];                                %K29JAN2020 - magnitude PDF/CDF 
    				catch err
	                HP(i).lamb_all=NaN; HP(i).lamb=NaN; HP(i).lamb_err=2; HP(i).unit=''; HP(i).eps=NaN; HP(i).b=NaN; HP(i).Mmax=NaN; HP(i).err=NaN;HP(i).PDF=NaN;HP(i).CDF=NaN;
	                HP(i).bCI(1)=NaN;HP(i).bCI(2)=NaN;warning('%s: %s', err.identifier, err.message);
				end
			case 'NPU'
				try
					[HP(i).lamb_all, HP(i).lamb, HP(i).lamb_err, HP(i).unit, HP(i).eps, HP(i).ierr, HP(i).h, HP(i).xx, HP(i).ambd]=Nonpar_O(time_vec, mags_vec, iop, mmin);
                    HP(i).Mmax=NaN;eps=HP(i).eps;
                    [m, PDF_NPU, CDF_NPU]=dist_NPU(mmin,Mmax+eps,eps,mmin,eps,HP(i).h,HP(i).xx,HP(i).ambd);              %K29JAN2020 - magnitude PDF/CDF
                    HP(i).PDF=[m PDF_NPU];HP(i).CDF=[m CDF_NPU];                                                         %K29JAN2020 - magnitude PDF/CDF 
                    data=[HP(i).xx,HP(i).ambd'];                                               % K04JUN2020 NPU
                    [CDF_NP]=dist_NPU_CI(mmin,Mmax+eps,eps,mmin,eps,HP(i).h,data);             % K04JUN2020 NPU
                    funct=@(data)dist_NPU_CI(mmin,Mmax+eps,eps,mmin,eps,HP(i).h,data);         % K04JUN2020 NPU
                    [CDF_CI]=bootci(Nbst,{funct,data},'alpha',0.05)';                          % K04JUN2020 NPU
                    HP(i).CDF=[HP(i).CDF CDF_CI];                                              % K04JUN2020 NPU
                catch err
	               HP(i).lamb_all=NaN; HP(i).lamb=NaN;HP(i).lamb_err=2; HP(i).unit=''; HP(i).eps=NaN; HP(i).ierr=NaN; HP(i).h=NaN; HP(i).xx=[]; HP(i).ambd=[];HP(i).Mmax=NaN;HP(i).PDF=NaN;HP(i).CDF=NaN;
	               warning('%s: %s', err.identifier, err.message);
				end
			case 'NPT'
				try
					[HP(i).lamb_all, HP(i).lamb, HP(i).lamb_err, HP(i).unit, HP(i).eps, HP(i).ierr, HP(i).h, HP(i).xx, HP(i).ambd, HP(i).Mmax, HP(i).err]=Nonpar_tr_O(time_vec, mags_vec, iop, mmin,Mmax);
                    eps=HP(i).eps;
                    [m, PDF_NPT, CDF_NPT]=dist_NPT(mmin,Mmax+eps,eps,mmin,eps,HP(i).h,HP(i).xx,HP(i).ambd,Mmax);         %K29JAN2020 - magnitude PDF/CDF
                    HP(i).PDF=[m PDF_NPT];HP(i).CDF=[m CDF_NPT];                                                         %K29JAN2020 - magnitude PDF/CDF 
                    data=[HP(i).xx,HP(i).ambd'];                                               % K04JUN2020 NPT
                    [CDF_NP]=dist_NPT_CI(mmin,Mmax+eps,eps,mmin,eps,HP(i).h,data,Mmax);        % K04JUN2020 NPT
                    funct=@(data)dist_NPT_CI(mmin,Mmax+eps,eps,mmin,eps,HP(i).h,data,Mmax);    % K04JUN2020 NPT
                    [CDF_CI]=bootci(Nbst,{funct,data},'alpha',0.05)';                          % K04JUN2020 NPT
                    HP(i).CDF=[HP(i).CDF CDF_CI];                                              % K04JUN2020 NPU
                catch err
	                HP(i).lamb_all=NaN; HP(i).lamb=NaN; HP(i).lamb_err=2; HP(i).unit=''; HP(i).eps=NaN; HP(i).ierr=NaN; HP(i).h=NaN; HP(i).xx=[]; HP(i).ambd=[]; HP(i).Mmax=NaN; HP(i).err=NaN;HP(i).PDF=NaN;HP(i).CDF=NaN;
	                warning('%s: %s', err.identifier, err.message);
				end
        end
  
   %   K12NOV2015
  % Calculate lamb and lamb_all in case of 0, 1, or 2 events. 
  % It may be generalized in all cases. Rate is now calculated
  % by division of the event number by the duration of the set 
  % - not the time difference between the first and last events.
   if numel(mags_vec(mags_vec>=mmin))<3  
    HP(i).lamb_all=NaN;   % numel(mags_vec)/(time_win_data(i).Tend-time_win_data(i).Tstart);    
    HP(i).lamb=NaN;       % numel(mags_vec(mags_vec>=mmin))/(time_win_data(i).Tend-time_win_data(i).Tstart);    
       switch iop
           case 0
            %OK   
           case 1
    HP(i).lamb_all=HP(i).lamb_all*30;
    HP(i).lamb=HP(i).lamb*30;
           case 2
    HP(i).lamb_all=HP(i).lamb_all*365;           
    HP(i).lamb=HP(i).lamb*365;
       end
   end   
 %   K12NOV2015

    end

cd ../
    
end

%% 
function [MRPer,ExPr,MRPer_low,MRPer_high,ExPr_low,ExPr_high]=RetPeriodExcProbWrapper(lamb,CDF,DT,MaG)
DM=abs(CDF(:,1)-MaG);ind=find(DM==min(DM));
CDF1=CDF(ind,2);CDF_CI=[CDF(ind,3) CDF(ind,4)];

% Exceedance probability
EP=1-exp(-lamb*DT.*(1-CDF1));
EP_CI=1-exp(-lamb*DT.*(1-CDF_CI));
ExPr=EP;ExPr_low=EP_CI(:,2);ExPr_high=EP_CI(:,1);
% Mean Return Period
MRP=1/lamb./(1-CDF1);
MRP_CI=1/lamb./(1-CDF_CI);
MRPer=MRP;MRPer_low=MRP_CI(:,1);MRPer_high=MRP_CI(:,2);

end
